@@ -20,7 +20,7 @@ import magic
 from os0 import os0
 from python_plus import _c
 
-__version__ = "1.0.10"
+__version__ = "2.0.2"
 
 # return code
 TEST_FAILED = 1
@@ -466,6 +466,8 @@ class Z0test(object):
         if self.autorun:
             self.ctx = self.parseoptest(argv, version=version)
             sys.exit(self.main())
+        self.home_devel = os.environ.get("HOME_DEVEL") or os.path.expanduser("~/devel")
+        self.odoo_root = os.path.dirname(self.home_devel)
 
     def _create_parser(self, version, ctx):
         """Standard test option parser; same funcionality of bash version
@@ -663,6 +665,19 @@ class Z0test(object):
         )
         return parser
 
+    def set_tlog_file(self, ctx):
+        # Old (deprecated) log management
+        os0.set_tlog_file(
+            ctx.get('logfn', None),
+            new=ctx.get('opt_new', False),
+            echo=ctx.get('opt_echo', False),
+        )
+        # Now set:
+        # - UDI (Unique DB Identifier): format "{pkgname}_{git_org}{major_version}"
+        # - UMLI (Unique Module Log Identifier):
+        #        format "{git_org}{major_version}.{repos}.{pkgname}"
+        pass
+
     def _create_params_dict(self, ctx):
         """Create all params dictionary"""
         ctx = self._create_def_params_dict(ctx)
@@ -696,7 +711,7 @@ class Z0test(object):
             else:
                 ctx['logfn'] = "~/" + ctx['this'] + ".log"
         if not ctx.get('WLOGCMD', None) and not ctx.get('_run_autotest', False):
-            os0.set_tlog_file(ctx['logfn'], new=ctx['opt_new'], echo=ctx['opt_echo'])
+            self.set_tlog_file(ctx)
         try:
             subprocess.call(
                 ['coverage', '--version'],
@@ -771,26 +786,6 @@ class Z0test(object):
         return ctx
 
     def _get_this_fqn(self):
-        # i = 1
-        # valid = False
-        # auto_this = False
-        # this_fqn = False
-        # while not valid and i < len(inspect.stack()):
-        #     this_fqn = os.path.abspath(inspect.stack()[i][1])
-        #     this = os.path.splitext(os.path.basename(this_fqn))[0]
-        #     if this[0] == '<' and this[-1] == '>':
-        #         i += 1
-        #     elif this in ("__init__", "pdb", "cmd", "z0testlib"):
-        #         i += 1
-        #         if this == "__init__":
-        #             auto_this = this_fqn
-        #     else:
-        #         valid = True
-        #         if this in ('pkgutil', 'runpy'):
-        #             this_fqn = os.path.dirname(auto_this)
-        #             id = 'test_%s.py' % os.path.basename(this_fqn)
-        #             this_fqn = os.path.join(this_fqn, id)
-        # return this_fqn
         return os.path.abspath(sys.argv[0])
 
     def parseoptest(self, arguments, version=None, tlog=None):
@@ -1179,19 +1174,10 @@ class Z0test(object):
             sts = TEST_SUCCESS
         else:
             if not ctx.get('_run_autotest', False):
-                os0.set_tlog_file(
-                    ctx.get('logfn', None),
-                    new=ctx.get('opt_new', False),
-                    echo=ctx.get('opt_echo', False),
-                )
+                self.set_tlog_file(ctx)
             sts = self._exec_all_tests(test_list, ctx, Test)
         return sts
 
-    #
-    # def main_file(self, ctx=None, Test=None, UT1=None, UT=None):
-    #     print('Deprecatede method: use main(..)')
-    #     self.main(ctx=ctx, Test=Test, UT1=UT1, UT=UT)
-
     def main(self, ctx=None, Test=None, UT1=None, UT=None):
         """Default main program for test execution
         ctx: context
@@ -1200,8 +1186,7 @@ class Z0test(object):
         UT1: protected Unit Test list (w/o log)
         UT: Unit Test list (if None, search for files)
         """
-        if ctx is None:
-            ctx = self.ctx
+        ctx = ctx or self.ctx
         if (
             ctx.get('opt_debug', False)
             and ctx.get('run_on_top', False)
@@ -1236,7 +1221,7 @@ class Z0test(object):
                     print('Coverage not found!')
                     ctx['run4cover'] = False
         test_list = []
-        if UT is not None and isinstance(UT, list):
+        if isinstance(UT, (list, tuple)):
             self.dbgmsg(ctx, '>>> test_list=%s' % UT)
             test_list = UT
         elif not ctx.get('_run_autotest', False):
@@ -1268,17 +1253,21 @@ class Z0test(object):
             sts = TEST_SUCCESS
         else:
             if not ctx.get('_run_autotest', False):
-                os0.set_tlog_file(
-                    ctx.get('logfn', None),
-                    new=ctx.get('opt_new', False),
-                    echo=ctx.get('opt_echo', False),
-                )
+                self.set_tlog_file(ctx)
             sts = self._exec_all_tests(test_list, ctx, Test)
             if ctx.get('run_on_top', False) and not ctx.get('_run_autotest', False):
                 if sts == TEST_SUCCESS:
                     print(success_msg)
                 else:
                     print(fail_msg)
+            if ctx.get('run4cover', False) and not ctx.get('dry_run', False):
+                try:
+                    subprocess.call(
+                        ['coverage', 'report', '--show-missing'],
+                    )
+                except OSError:
+                    print('Coverage not found!')
+                    ctx['run4cover'] = False
         return sts
 
     def dbgmsg(self, ctx, msg, echo=None):
